var newButton, openButton, saveButton;
var editor;
var menu;
var fileEntry;
var hasWriteAccess;

// requires lingo libs json and base64
lingo = {
	call: function(func, obj, args, callback){
		var req = 'return $.json.encode(call(#'+func+','+obj;
		for (var i in args) {
			req += ',$.json.decode($.base64.decode("'+window.btoa(JSON.stringify(args[i]))+'"))';
		}
		req += '))';
		window.cefQuery({
			request: req,
			persistent: false,
			onSuccess: function(response){callback(JSON.parse(response));},
			onFailure: function(error_code, error_message) {
				console.log("Lingo call failed: " + error_code + ', ' + error_message);
			}
	    });
	}
}

function handleDocumentChange(title) {
	var mode = "javascript";
	var modeName = "JavaScript";
	if (title) {
		title = title.match(/[^/]+$/)[0];
		document.getElementById("title").innerHTML = title;
		document.title = title;
		if (title.match(/.json$/)) {
			mode = {name: "javascript", json: true};
			modeName = "JavaScript (JSON)";
		} else if (title.match(/.html$/)) {
			mode = "htmlmixed";
			modeName = "HTML";
		} else if (title.match(/.css$/)) {
			mode = "css";
			modeName = "CSS";
		}
	} else {
		document.getElementById("title").innerHTML = "[no document loaded]";
	}
	editor.setOption("mode", mode);
	document.getElementById("mode").innerHTML = modeName;
}

function newFile() {
	fileEntry = null;
	hasWriteAccess = false;
	handleDocumentChange(null);
}

function setFile(theFileEntry, isWritable) {
	fileEntry = theFileEntry;
	hasWriteAccess = isWritable;
}

function readFileIntoEditor(theFileEntry) {
	lingo.call('getString', '$.file', [theFileEntry], function(response){
		if (typeof response=='string'){
			handleDocumentChange(theFileEntry);
			editor.setValue(response);
		}else{
			console.log("Read failed");
		}
	})
}

function writeEditorToFile(theFileEntry) {
	lingo.call('putString', '$.file', [theFileEntry, editor.getValue()], function(response){
		if (response==1){
			handleDocumentChange(theFileEntry);
			console.log("Write completed.");
		}else{
			console.log("Write failed");
		}
	})
}

function onChosenFileToOpen(theFileEntry) {
	setFile(theFileEntry, true); // false
	readFileIntoEditor(theFileEntry);
};
	
function onChosenFileToSave(theFileEntry) {
	setFile(theFileEntry, true);
	writeEditorToFile(theFileEntry);
};

function handleNewButton() {
	newFile();
	editor.setValue("");
}

function handleOpenButton() {
	lingo.call('selectFileOpen', '$.file', ["Open file...", "", ""], function(response){
		if (typeof response=='string' && response!='') {
			onChosenFileToOpen(response);
		}
	})
}

function handleSaveButton() {
	if (fileEntry && hasWriteAccess) {
		writeEditorToFile(fileEntry);
	} else {
		lingo.call('selectFileSave', '$.file', ["Save file as...", "", ""], function(response){
			if (typeof response=='string' && response!='') {
				onChosenFileToSave(response, true);
			}
		})
	}
}

onload = function() {
	newButton = document.getElementById("new");
	openButton = document.getElementById("open");
	saveButton = document.getElementById("save");

	newButton.addEventListener("click", handleNewButton);
	openButton.addEventListener("click", handleOpenButton);
	saveButton.addEventListener("click", handleSaveButton);

	editor = CodeMirror(document.getElementById("editor"),
	{
		mode: {name: "javascript", json: true },
		lineNumbers: true,
		theme: "lesser-dark",
		extraKeys: {
			"Cmd-S": function(instance) { handleSaveButton() },
			"Ctrl-S": function(instance) { handleSaveButton() },
		}
	});

	newFile();
	onresize();
};

onresize = function() {
	var container = document.getElementById('editor');
	var containerWidth = container.offsetWidth;
	var containerHeight = container.offsetHeight;

	var scrollerElement = editor.getScrollerElement();
	scrollerElement.style.width = containerWidth + 'px';
	scrollerElement.style.height = containerHeight + 'px';

	editor.refresh();
}
