--!parent
--!encoding=utf-8

--****************************************************************************
-- @file LSW CORE LIB: JSON
-- @author Valentin Schmidt
-- @version 0.2
--****************************************************************************

property $
property jsHelper

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	jsHelper = $.include(me.libdir&"json.js") -- JS helper functions
	return me
end

----------------------------------------
-- JSON_DECODE
-- @param {string|byteArray} data
-- @return {any}
----------------------------------------
on decode (me, data, symbolProps)
	if ilk(data)=#byteArray then
		data.position = 1
		data = data.readRawString(data.length)
	end if
	--_global.data = data
	return jsHelper.json_decode(data, symbolProps)
end

----------------------------------------
-- JSON_ENCODE
-- @param {any} c
-- @return {any}
----------------------------------------
on encode (me, o)
	case ilk(o) of
		#void:
			return "null"
		#integer, #float:
			return string(o)
		#string:
			o = me._replace("\","\\",o)
			return QUOTE & jsHelper.json_escape_string(o) & QUOTE
		#list:
			res = []
			repeat with v in o
				res.add(me.encode(v))
			end repeat
			return "[" & me._implode(",", res) & "]"
		#propList:
			res = []
			cnt = count(o)
			repeat with i = 1 to cnt
				p = o.getPropAt(i)
				v = o[i]
				res.add( me.encode(p)&":"&me.encode(v) )
			end repeat
			return "{" & me._implode(",", res) & "}"
		#symbol:
			--return QUOTE&"_sym_"&string(o)&QUOTE
			o = me._replace("\","\\",o)
			return QUOTE & jsHelper.json_escape_string(string(o)) & QUOTE

		otherwise:
			-- put "ERROR: unsupported data type!"
			return QUOTE&"<UNSUPPORTED DATA TYPE: "&ilk(o)&">"&QUOTE

	end case
end

----------------------------------------
--
----------------------------------------
on streamParse (me, fn, parseLevel, rowCallbackHandler, rowCallbackTarget)
	if voidP(rowCallbackTarget) then rowCallbackTarget = _movie

	$.import("file")

	fp = $.file.fopen(fn, "r")
	fileLen = $.file.fsize(fp)

	-- check for BOM 0xef, 0xbb, 0xbf
	tmp = $.file.freadbytes(fp, 3)
	if tmp[1] = 239 then
		fileLen = fileLen-3
	else
		$.file.fseek(fp, 0)
	end if

	-- read bufSize bytes into byteArray
	ba = $.file.freadbytes(fp, fileLen)
	--put "FILE LOADED", ba.length

	lastPos = 1
	first = 1
	stack = []
	quoted = FALSE
	lastChar = 0

	repeat with j = 1 to fileLen
		c = ba[j]

		if c=34 and lastChar<>92 then -- "	backslash
			quoted = not quoted

		else if not quoted then

			if c=123 or c=91 then -- {	[
				--put "open", stack.count
				stack.add(j)

			else if c=125 or c=93 then -- }	]
				--put "close"

				lastPos = stack.getLast()
				stack.deleteAt(stack.count)
				ba.position = lastPos

				--if parseLevels.getPos(stack.count) then
				if parseLevel=stack.count then
					val = me.decode(ba.readRawString(j-lastPos+1))
					if not voidP(val) then
						ok = call(rowCallbackHandler, rowCallbackTarget, val) --, stack.count)
						if ok=FALSE then return
					end if
				end if
			end if

		end if

		lastChar = c
	end repeat
end

----------------------------------------
--
----------------------------------------
on _implode (me, delim, l)
	str=""
	repeat with i=1 to l.count
		put l[i]&delim after str
	end repeat
	delete char (str.length-delim.length+1) to str.length of str
	return str
end

on _replace (me, stringToFind, stringToInsert, input)
	output = ""
	findLen = stringToFind.length - 1
	repeat while true
		currOffset = offset(stringToFind, input)
		if currOffset=0 then exit repeat
		put input.char [1..currOffset] after output
		delete the last char of output
		put stringToInsert after output
		delete input.char [1.. (currOffset + findLen)]
	end repeat
	put input after output
	return output
end