--!parent
--!encoding=utf-8

--****************************************************************************
-- @file LSW CORE LIB: MSG
-- @author Valentin Schmidt
-- @version 0.5
-- @requires xtra("Msg")
--****************************************************************************

property $

property ancestor

property _watchDirHandler
property _watchDirTarget

property _msgListenHandler
property _msgListenTarget

----------------------------------------
-- @constructor
----------------------------------------
on new (me)

	-- libs
	$.import("detacher")

	$.loadXtras(me.libdir, ["Msg.x32"], [])
	--put xtra("msg").interface()

	me.ancestor = xtra("Msg").new()

	-- load constants (optional)
	me.loadConstantsGW()
	me.loadConstantsSMTO()
	me.loadConstantsWM()
	me.loadConstantsNotifyFilter()
	me.loadConstantsSW()
	me.loadConstantsSM()
	me.loadConstantsWS()

	return me
end

----------------------------------------
-- Watch for directory changes
-- @param {string} dir
-- @param {integer} notifyFilter
-- @param {symbol} cbHandler
-- @param {object} [cbTarget=_movie]
----------------------------------------
on watch_dir (me, dir, notifyFilter, cbHandler, cbTarget)
	if voidp(cbTarget) then cbTarget = _movie

	me._watchDirHandler = cbHandler
	me._watchDirTarget = cbTarget

	me.ancestor.watch_dir(dir, notifyFilter, #_watchDirCallback, me)
end

----------------------------------------
-- @callback
----------------------------------------
on _watchDirCallback (me, fn, changeAction)
	if symbolP(me._watchDirHandler) then
		me.$.detacher.detach(me._watchDirHandler, me._watchDirTarget, fn, changeAction)
	end if
end

----------------------------------------
-- Listen to Win messages
-- msg_listen object me, object msgList, any blockList, symbol cbHandler, *cbTarget
-- @param {list} msgList
-- @param {list|VOID} blockList
-- @param {symbol} cbHandler
-- @param {object} [cbTarget=_movie]
----------------------------------------
on msg_listen (me, msgList, blockList, cbHandler, cbTarget)
	if voidp(cbTarget) then cbTarget = _movie

	me._msgListenHandler = cbHandler
	me._msgListenTarget = cbTarget

	me.ancestor.msg_listen(msgList, blockList, #_msgListenCallback, me)
end

----------------------------------------
-- @callback
----------------------------------------
on _msgListenCallback (me, hwnd, msg, wParam, lParam, data, dwData)
	if symbolP(me._msgListenHandler) then
		me.$.detacher.detach(me._msgListenHandler, me._msgListenTarget, [hwnd, msg, wParam, lParam, data, dwData])
	end if
end

----------------------------------------
-- GetWindow() Constants
----------------------------------------
on loadConstantsGW (me)
	me["GW_HWNDFIRST"] = 0
	me["GW_HWNDLAST"] = 1
	me["GW_HWNDNEXT"] = 2
	me["GW_HWNDPREV"] = 3
	me["GW_OWNER"] = 4
	me["GW_CHILD"] = 5
end

----------------------------------------
-- SendMessageTimeout values
----------------------------------------
on loadConstantsSMTO (me)
	me["SMTO_NORMAL"] = 0
	me["SMTO_BLOCK"] = 1
	me["SMTO_ABORTIFHUNG"] = 2
	--#if(WINVER >= 0x0500)
	me["SMTO_NOTIMEOUTIFNOTHUNG"] = 8
	--#endif
end

----------------------------------------
-- Window Messages
----------------------------------------
on loadConstantsWM (me)
	-- custom message defined by Director
	me["WM_SENDSCRIPT"] = 1579 -- WM_USER+555

	me["WM_MOUSEWHEEL"] = 522

	me["WM_ACTIVATE"] = 6
	me["WM_ACTIVATEAPP"] = 28
	me["WM_ASKCBFORMATNAME"] = 780
	me["WM_CANCELJOURNAL"] = 75
	me["WM_CANCELMODE"] = 31
	me["WM_CAPTURECHANGED"] = 533
	me["WM_CHANGECBCHAIN"] = 781
	me["WM_CHAR"] = 258
	me["WM_CHARTOITEM"] = 47
	me["WM_CHILDACTIVATE"] = 34
	me["WM_CHOOSEFONT_GETLOGFONT"] = 1025
	me["WM_CHOOSEFONT_SETFLAGS"] = 1126
	me["WM_CHOOSEFONT_SETLOGFONT"] = 1125
	me["WM_CLEAR"] = 771
	me["WM_CLOSE"] = 16
	me["WM_COMMAND"] = 273
	me["WM_COMPACTING"] = 67
	me["WM_COMPAREITEM"] = 57
	me["WM_CONTEXTMENU"] = 123
	me["WM_COPY"] = 769
	me["WM_COPYDATA"] = 74
	me["WM_CREATE"] = 1
	me["WM_CTLCOLORBTN"] = 309
	me["WM_CTLCOLORDLG"] = 310
	me["WM_CTLCOLOREDIT"] = 307
	me["WM_CTLCOLORLISTBOX"] = 308
	me["WM_CTLCOLORMSGBOX"] = 306
	me["WM_CTLCOLORSCROLLBAR"] = 311
	me["WM_CTLCOLORSTATIC"] = 312
	me["WM_CUT"] = 768
	me["WM_DDE_ACK"] = 926
	me["WM_DDE_ADVISE"] = 924
	me["WM_DDE_DATA"] = 927
	me["WM_DDE_EXECUTE"] = 930
	me["WM_DDE_FIRST"] = 922
	me["WM_DDE_INITIATE"] = 922
	me["WM_DDE_LAST"] = 930
	me["WM_DDE_POKE"] = 929
	me["WM_DDE_REQUEST"] = 928
	me["WM_DDE_TERMINATE"] = 923
	me["WM_DDE_UNADVISE"] = 925
	me["WM_DEADCHAR"] = 259
	me["WM_DELETEITEM"] = 45
	me["WM_DESTROY"] = 2
	me["WM_DESTROYCLIPBOARD"] = 775
	me["WM_DEVICECHANGE"] = 537
	me["WM_DEVMODECHANGE"] = 27
	me["WM_DISPLAYCHANGE"] = 126
	me["WM_DRAWCLIPBOARD"] = 776
	me["WM_DRAWITEM"] = 43
	me["WM_DROPFILES"] = 563
	me["WM_ENABLE"] = 10
	me["WM_ENDSESSION"] = 22
	me["WM_ENTERIDLE"] = 289
	me["WM_ENTERMENULOOP"] = 529
	me["WM_ENTERSIZEMOVE"] = 561
	me["WM_ERASEBKGND"] = 20
	me["WM_EXITMENULOOP"] = 530
	me["WM_EXITSIZEMOVE"] = 562
	me["WM_FONTCHANGE"] = 29
	me["WM_GETDLGCODE"] = 135
	me["WM_GETFONT"] = 49
	me["WM_GETHOTKEY"] = 51
	me["WM_GETICON"] = 127
	me["WM_GETMINMAXINFO"] = 36
	me["WM_GETTEXT"] = 13
	me["WM_GETTEXTLENGTH"] = 14
	me["WM_HELP"] = 83
	me["WM_HOTKEY"] = 786
	me["WM_HSCROLL"] = 276
	me["WM_HSCROLLCLIPBOARD"] = 782
	me["WM_ICONERASEBKGND"] = 39
	me["WM_IME_CHAR"] = 646
	me["WM_IME_COMPOSITION"] = 271
	me["WM_IME_COMPOSITIONFULL"] = 644
	me["WM_IME_CONTROL"] = 643
	me["WM_IME_ENDCOMPOSITION"] = 270
	me["WM_IME_KEYDOWN"] = 656
	me["WM_IME_KEYUP"] = 657
	me["WM_IME_NOTIFY"] = 642
	me["WM_IME_SELECT"] = 645
	me["WM_IME_SETCONTEXT"] = 641
	me["WM_IME_STARTCOMPOSITION"] = 269
	me["WM_INITDIALOG"] = 272
	me["WM_INITMENU"] = 278
	me["WM_INITMENUPOPUP"] = 279
	me["WM_INPUTLANGCHANGE"] = 81
	me["WM_INPUTLANGCHANGEREQUEST"] = 80
	me["WM_KEYDOWN"] = 256
	me["WM_KEYFIRST"] = 256
	me["WM_KEYLAST"] = 264
	me["WM_KEYUP"] = 257
	me["WM_KILLFOCUS"] = 8
	me["WM_LBUTTONDBLCLK"] = 515
	me["WM_LBUTTONDOWN"] = 513
	me["WM_LBUTTONUP"] = 514
	me["WM_MBUTTONDBLCLK"] = 521
	me["WM_MBUTTONDOWN"] = 519
	me["WM_MBUTTONUP"] = 520
	me["WM_MDIACTIVATE"] = 546
	me["WM_MDICASCADE"] = 551
	me["WM_MDICREATE"] = 544
	me["WM_MDIDESTROY"] = 545
	me["WM_MDIGETACTIVE"] = 553
	me["WM_MDIICONARRANGE"] = 552
	me["WM_MDIMAXIMIZE"] = 549
	me["WM_MDINEXT"] = 548
	me["WM_MDIREFRESHMENU"] = 564
	me["WM_MDIRESTORE"] = 547
	me["WM_MDISETMENU"] = 560
	me["WM_MDITILE"] = 550
	me["WM_MEASUREITEM"] = 44
	me["WM_MENUCHAR"] = 288
	me["WM_MENUSELECT"] = 287
	me["WM_MOUSEACTIVATE"] = 33
	me["WM_MOUSEFIRST"] = 512
	me["WM_MOUSELAST"] = 521
	me["WM_MOUSEMOVE"] = 512
	me["WM_MOVE"] = 3
	me["WM_MOVING"] = 534
	me["WM_NCACTIVATE"] = 134
	me["WM_NCCALCSIZE"] = 131
	me["WM_NCCREATE"] = 129
	me["WM_NCDESTROY"] = 130
	me["WM_NCHITTEST"] = 132
	me["WM_NCLBUTTONDBLCLK"] = 163
	me["WM_NCLBUTTONDOWN"] = 161
	me["WM_NCLBUTTONUP"] = 162
	me["WM_NCMBUTTONDBLCLK"] = 169
	me["WM_NCMBUTTONDOWN"] = 167
	me["WM_NCMBUTTONUP"] = 168
	me["WM_NCMOUSEMOVE"] = 160
	me["WM_NCPAINT"] = 133
	me["WM_NCRBUTTONDBLCLK"] = 166
	me["WM_NCRBUTTONDOWN"] = 164
	me["WM_NCRBUTTONUP"] = 165
	me["WM_NEXTDLGCTL"] = 40
	me["WM_NOTIFY"] = 78
	me["WM_NOTIFYFORMAT"] = 85
	me["WM_NULL"] = 0
	me["WM_PAINT"] = 15
	me["WM_PAINTCLIPBOARD"] = 777
	me["WM_PAINTICON"] = 38
	me["WM_PALETTECHANGED"] = 785
	me["WM_PALETTEISCHANGING"] = 784
	me["WM_PARENTNOTIFY"] = 528
	me["WM_PASTE"] = 770
	me["WM_PENWINFIRST"] = 896
	me["WM_PENWINLAST"] = 911
	me["WM_POWER"] = 72
	me["WM_POWERBROADCAST"] = 536
	me["WM_PRINT"] = 791
	me["WM_PRINTCLIENT"] = 792
	me["WM_PSD_ENVSTAMPRECT"] = 1029
	me["WM_PSD_FULLPAGERECT"] = 1025
	me["WM_PSD_GREEKTEXTRECT"] = 1028
	me["WM_PSD_MARGINRECT"] = 1027
	me["WM_PSD_MINMARGINRECT"] = 1026
	me["WM_PSD_PAGESETUPDLG"] = 1024
	me["WM_PSD_YAFULLPAGERECT"] = 1030
	me["WM_QUERYDRAGICON"] = 55
	me["WM_QUERYENDSESSION"] = 17
	me["WM_QUERYNEWPALETTE"] = 783
	me["WM_QUERYOPEN"] = 19
	me["WM_QUEUESYNC"] = 35
	me["WM_QUIT"] = 18
	me["WM_RBUTTONDBLCLK"] = 518
	me["WM_RBUTTONDOWN"] = 516
	me["WM_RBUTTONUP"] = 517
	me["WM_RENDERALLFORMATS"] = 774
	me["WM_RENDERFORMAT"] = 773
	me["WM_SETCURSOR"] = 32
	me["WM_SETFOCUS"] = 7
	me["WM_SETFONT"] = 48
	me["WM_SETHOTKEY"] = 50
	me["WM_SETICON"] = 128
	me["WM_SETREDRAW"] = 11
	me["WM_SETTEXT"] = 12
	me["WM_SETTINGCHANGE"] = 26
	me["WM_SHOWWINDOW"] = 24
	me["WM_SIZE"] = 5
	me["WM_SIZECLIPBOARD"] = 779
	me["WM_SIZING"] = 532
	me["WM_SPOOLERSTATUS"] = 42
	me["WM_STYLECHANGED"] = 125
	me["WM_STYLECHANGING"] = 124
	me["WM_SYSCHAR"] = 262
	me["WM_SYSCOLORCHANGE"] = 21
	me["WM_SYSCOMMAND"] = 274
	me["WM_SYSDEADCHAR"] = 263
	me["WM_SYSKEYDOWN"] = 260
	me["WM_SYSKEYUP"] = 261
	me["WM_TCARD"] = 82
	me["WM_TIMECHANGE"] = 30
	me["WM_TIMER"] = 275
	me["WM_TOUCH"] = 576
	me["WM_UNDO"] = 772
	me["WM_USER"] = 1024
	me["WM_USERCHANGED"] = 84
	me["WM_VKEYTOITEM"] = 46
	me["WM_VSCROLL"] = 277
	me["WM_VSCROLLCLIPBOARD"] = 778
	me["WM_WINDOWPOSCHANGED"] = 71
	me["WM_WINDOWPOSCHANGING"] = 70
	me["WM_WININICHANGE"] = 26
end

----------------------------------------
-- NotifyFilter
----------------------------------------
on loadConstantsNotifyFilter (me)

	-- dwNotifyFilter
	-- https://msdn.microsoft.com/en-us/library/windows/desktop/aa365465(v=vs.85).aspx

	--Any file name change in the watched directory or subtree causes a change
	--notification wait operation to return. Changes include renaming, creating,
	--or deleting a file name.
	me["FILE_NOTIFY_CHANGE_FILE_NAME"] = 1

	--Any directory-name change in the watched directory or subtree causes a change
	--notification wait operation to return. Changes include creating or deleting a
	--directory.
	me["FILE_NOTIFY_CHANGE_DIR_NAME"] = 2

	--Any attribute change in the watched directory or subtree causes a change
	--notification wait operation to return.
	me["FILE_NOTIFY_CHANGE_ATTRIBUTES"] = 4

	--Any file-size change in the watched directory or subtree causes a change
	--notification wait operation to return. The operating system detects a change in
	--file size only when the file is written to the disk. For operating systems that
	--use extensive caching, detection occurs only when the cache is sufficiently flushed.
	me["FILE_NOTIFY_CHANGE_SIZE"] = 8

	--Any change to the last write-time of files in the watched directory or subtree
	--causes a change notification wait operation to return. The operating system detects
	--a change to the last write-time only when the file is written to the disk. For
	--operating systems that use extensive caching, detection occurs only when the cache
	--is sufficiently flushed.
	me["FILE_NOTIFY_CHANGE_LAST_WRITE"] = 16

	--Any security-descriptor change in the watched directory or subtree causes a
	--change notification wait operation to return.
	me["FILE_NOTIFY_CHANGE_SECURITY"] = 256

	-- ACTIONS RETURNED
	-- https://msdn.microsoft.com/de-de/library/windows/desktop/aa364391(v=vs.85).aspx
	me["FILE_ACTION_ADDED"] = 1 -- The file was added to the directory.
	me["FILE_ACTION_REMOVED"] = 2 -- The file was removed from the directory.
	me["FILE_ACTION_MODIFIED"] = 3 -- The file was modified. This can be a change in the time stamp or attributes.
	me["FILE_ACTION_RENAMED_OLD_NAME"] = 4 -- The file was renamed and this is the old name
	me["FILE_ACTION_RENAMED_NEW_NAME"] = 5 -- The file was renamed and this is the new name.
end

----------------------------------------
-- ShowWindow constants
----------------------------------------
on loadConstantsSW (me)
	me["SW_HIDE"] = 0
	me["SW_SHOWNORMAL"] = 1
	me["SW_SHOWMINIMIZED"] = 2
	me["SW_MAXIMIZE"] = 3
	me["SW_SHOWMAXIMIZED"] = 3
	me["SW_SHOWNOACTIVATE"] = 4
	me["SW_SHOW"] = 5
	me["SW_MINIMIZE"] = 6
	me["SW_SHOWMINNOACTIVE"] = 7
	me["SW_SHOWNA"] = 8
	me["SW_RESTORE"] = 9
	me["SW_SHOWDEFAULT"] = 10
	me["SW_FORCEMINIMIZE"] = 11
end

----------------------------------------
-- SystemMetrics constants (https://msdn.microsoft.com/en-us/library/windows/desktop/ms724385(v=vs.85).aspx)
----------------------------------------
on loadConstantsSM (me)
	me["SM_ARRANGE"] = 56
	me["SM_CLEANBOOT"] = 67
	me["SM_CMONITORS"] = 80
	me["SM_CMOUSEBUTTONS"] = 43
	me["SM_CONVERTIBLESLATEMODE"] = 8195
	me["SM_CXBORDER"] = 5
	me["SM_CXCURSOR"] = 13
	me["SM_CXDLGFRAME"] = 7
	me["SM_CXDOUBLECLK"] = 36
	me["SM_CXDRAG"] = 68
	me["SM_CXEDGE"] = 45
	me["SM_CXFIXEDFRAME"] = 7
	me["SM_CXFOCUSBORDER"] = 83
	me["SM_CXFRAME"] = 32
	me["SM_CXFULLSCREEN"] = 16
	me["SM_CXHSCROLL"] = 21
	me["SM_CXHTHUMB"] = 10
	me["SM_CXICON"] = 11
	me["SM_CXICONSPACING"] = 38
	me["SM_CXMAXIMIZED"] = 61
	me["SM_CXMAXTRACK"] = 59
	me["SM_CXMENUCHECK"] = 71
	me["SM_CXMENUSIZE"] = 54
	me["SM_CXMIN"] = 28
	me["SM_CXMINIMIZED"] = 57
	me["SM_CXMINSPACING"] = 47
	me["SM_CXMINTRACK"] = 34
	me["SM_CXPADDEDBORDER"] = 92
	me["SM_CXSCREEN"] = 0
	me["SM_CXSIZE"] = 30
	me["SM_CXSIZEFRAME"] = 32
	me["SM_CXSMICON"] = 49
	me["SM_CXSMSIZE"] = 52
	me["SM_CXVIRTUALSCREEN"] = 78
	me["SM_CXVSCROLL"] = 2
	me["SM_CYBORDER"] = 6
	me["SM_CYCAPTION"] = 4
	me["SM_CYCURSOR"] = 14
	me["SM_CYDLGFRAME"] = 8
	me["SM_CYDOUBLECLK"] = 37
	me["SM_CYDRAG"] = 69
	me["SM_CYEDGE"] = 46
	me["SM_CYFIXEDFRAME"] = 8
	me["SM_CYFOCUSBORDER"] = 84
	me["SM_CYFRAME"] = 33
	me["SM_CYFULLSCREEN"] = 17
	me["SM_CYHSCROLL"] = 3
	me["SM_CYICON"] = 12
	me["SM_CYICONSPACING"] = 39
	me["SM_CYKANJIWINDOW"] = 18
	me["SM_CYMAXIMIZED"] = 62
	me["SM_CYMAXTRACK"] = 60
	me["SM_CYMENU"] = 15
	me["SM_CYMENUCHECK"] = 72
	me["SM_CYMENUSIZE"] = 55
	me["SM_CYMIN"] = 29
	me["SM_CYMINIMIZED"] = 58
	me["SM_CYMINSPACING"] = 48
	me["SM_CYMINTRACK"] = 35
	me["SM_CYSCREEN"] = 1
	me["SM_CYSIZE"] = 31
	me["SM_CYSIZEFRAME"] = 33
	me["SM_CYSMCAPTION"] = 51
	me["SM_CYSMICON"] = 50
	me["SM_CYSMSIZE"] = 53
	me["SM_CYVIRTUALSCREEN"] = 79
	me["SM_CYVSCROLL"] = 20
	me["SM_CYVTHUMB"] = 9
	me["SM_DBCSENABLED"] = 42
	me["SM_DEBUG"] = 22
	me["SM_DIGITIZER"] = 94
	me["SM_IMMENABLED"] = 82
	me["SM_MAXIMUMTOUCHES"] = 95
	me["SM_MEDIACENTER"] = 87
	me["SM_MENUDROPALIGNMENT"] = 40
	me["SM_MIDEASTENABLED"] = 74
	me["SM_MOUSEPRESENT"] = 19
	me["SM_MOUSEHORIZONTALWHEELPRESENT"] = 91
	me["SM_MOUSEWHEELPRESENT"] = 75
	me["SM_NETWORK"] = 63
	me["SM_PENWINDOWS"] = 41
	me["SM_REMOTECONTROL"] = 8193
	me["SM_REMOTESESSION"] = 4096
	me["SM_SAMEDISPLAYFORMAT"] = 81
	me["SM_SECURE"] = 44
	me["SM_SERVERR2"] = 89
	me["SM_SHOWSOUNDS"] = 70
	me["SM_SHUTTINGDOWN"] = 8192
	me["SM_SLOWMACHINE"] = 73
	me["SM_STARTER"] = 88
	me["SM_SWAPBUTTON"] = 23
	me["SM_SYSTEMDOCKED"] = 8196
	me["SM_TABLETPC"] = 86
	me["SM_XVIRTUALSCREEN"] = 76
end
----------------------------------------
-- WindowStyle constants (https://docs.microsoft.com/de-de/windows/desktop/winmsg/window-styles)
----------------------------------------
on loadConstantsWS (me)
	me[#WS_BORDER] = 8388608
	me[#WS_CAPTION] = 12582912
	me[#WS_CHILD] = 1073741824
	me[#WS_CHILDWINDOW] = 1073741824
	me[#WS_CLIPCHILDREN] = 33554432
	me[#WS_CLIPSIBLINGS] = 67108864
	me[#WS_DISABLED] = 134217728
	me[#WS_DLGFRAME] = 4194304
	me[#WS_GROUP] = 131072
	me[#WS_HSCROLL] = 1048576
	me[#WS_ICONIC] = 536870912
	me[#WS_MAXIMIZE] = 16777216
	me[#WS_MAXIMIZEBOX] = 65536
	me[#WS_MINIMIZE] = 536870912
	me[#WS_MINIMIZEBOX] = 131072
	me[#WS_OVERLAPPED] = 0
	me[#WS_POPUP] = 2147483648
	me[#WS_SIZEBOX] = 262144
	me[#WS_SYSMENU] = 524288
	me[#WS_TABSTOP] = 65536
	me[#WS_THICKFRAME] = 262144
	me[#WS_TILED] = 0
	me[#WS_VISIBLE] = 268435456
	me[#WS_VSCROLL] = 2097152

	--WS_OVERLAPPEDWINDOW = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX
	me[#WS_OVERLAPPEDWINDOW] = 13565952

	--WS_POPUPWINDOW = WS_POPUP | WS_BORDER | WS_SYSMENU
	me[#WS_POPUPWINDOW] = -2138570752

	--WS_TILEDWINDOW = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MINIMIZEBOX | WS_MAXIMIZEBOX
	me[#WS_TILEDWINDOW] = 13565952
end
